@@ -1,11 +1,11 @@
 use strict;
 use warnings;
-package Test::Warnings; # git description: v0.026-14-g4e3ad06
+package Test::Warnings; # git description: v0.028-5-ged07d93
 # vim: set ts=8 sts=4 sw=4 tw=115 et :
 # ABSTRACT: Test for warnings and the lack of them
 # KEYWORDS: testing tests warnings
 
-our $VERSION = '0.027';
+our $VERSION = '0.029';
 
 use parent 'Exporter';
 use Test::Builder;
@@ -22,6 +22,8 @@ my $forbidden_warnings_found;
 my $done_testing_called;
 my $no_end_test;
 my $fail_on_warning;
+my $report_warnings;
+my @collected_warnings;
 
 sub import
 {
@@ -32,8 +34,10 @@ sub import
     $no_end_test = exists $names{':no_end_test'};
     # __WARN__ handler will check for this status
     $fail_on_warning = exists $names{':fail_on_warning'};
+    # Collect and report warnings at the end
+    $report_warnings = exists $names{':report_warnings'};
 
-    delete @names{qw(:no_end_test :fail_on_warning)};
+    delete @names{qw(:no_end_test :fail_on_warning :report_warnings)};
     __PACKAGE__->export_to_level(1, $class, keys %names);
 }
 
@@ -59,6 +63,7 @@ $SIG{__WARN__} = sub {
     else
     {
         $forbidden_warnings_found++;
+        push @collected_warnings, $_[0] if $report_warnings;
 
         # TODO: this doesn't handle blessed coderefs... does anyone care?
         goto &$_orig_warn_handler if $_orig_warn_handler
@@ -77,8 +82,14 @@ $SIG{__WARN__} = sub {
     }
 };
 
-sub warnings(&)
+sub warnings(;&)
 {
+    # if someone manually does warnings->import in the same namespace this is
+    # imported into, this sub will be called.  in that case, just return the
+    # string "warnings" so it calls the correct method.
+    if (!@_) {
+        return 'warnings';
+    }
     my $code = shift;
     my @warnings;
     local $SIG{__WARN__} = sub {
@@ -145,6 +156,12 @@ sub allowing_warnings() { $warnings_allowed }
 sub had_no_warnings(;$)
 {
     _builder->ok(!$forbidden_warnings_found, shift || 'no (unexpected) warnings');
+    if ($report_warnings and $forbidden_warnings_found) {
+        _builder->diag("Got the following unexpected warnings:");
+        for my $i (1 .. @collected_warnings) {
+            _builder->diag("  $i: $collected_warnings[ $i - 1 ]");
+        }
+    }
 }
 
 1;
@@ -161,7 +178,7 @@ Test::Warnings - Test for warnings and the lack of them
 
 =head1 VERSION
 
-version 0.027
+version 0.029
 
 =head1 SYNOPSIS
 
@@ -346,6 +363,11 @@ L</had_no_warnings> or C<done_testing> is called).
 I recommend you only turn this option on when debugging a test, to see where a surprise warning is coming from,
 and rely on the end-of-tests check otherwise.
 
+=head2 C<:report_warnings>
+
+When used, C<had_no_warnings()> will print all the unexempted warning content, in case it had been suppressed
+earlier by other captures (such as L<Test::Output/stderr_like> or L<Capture::Tiny/capture>).
+
 =head1 CAVEATS
 
 =for stopwords smartmatch TODO irc
@@ -453,22 +475,26 @@ Karen Etheridge <ether@cpan.org>
 
 =head1 CONTRIBUTORS
 
-=for stopwords A. Sinan Unur Graham Knop Leon Timmermans
+=for stopwords Graham Knop A. Sinan Unur Leon Timmermans Tina Mueller
 
 =over 4
 
 =item *
 
-A. Sinan Unur <nanis@cpan.org>
-
-=item *
-
 Graham Knop <haarg@haarg.org>
 
 =item *
 
+A. Sinan Unur <nanis@cpan.org>
+
+=item *
+
 Leon Timmermans <fawaka@gmail.com>
 
+=item *
+
+Tina Mueller <cpan2@tinita.de>
+
 =back
 
 =head1 COPYRIGHT AND LICENCE
